/**
 * Copyright 2015-2017 Red Hat, Inc, and individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wildfly.swarm.jaxrs.internal;

import javax.enterprise.context.ApplicationScoped;

import org.jboss.shrinkwrap.api.Archive;
import org.wildfly.swarm.jaxrs.JAXRSArchive;
import org.wildfly.swarm.undertow.internal.DefaultWarDeploymentFactory;

/**
 * @author Bob McWhirter
 */
@ApplicationScoped
public class DefaultJAXRSWarDeploymentFactory extends DefaultWarDeploymentFactory {

    @Override
    public int getPriority() {
        return 1000;
    }

    @Override
    public String getType() {
        return "war";
    }

    @Override
    public Archive create() throws Exception {
        Archive archive = super.create();
        // SWARM-917: only implicitly consider it a JAXRS archive if it happens to actually be one,
        // in order to prevent gratuitous creation of @ApplicationPath stuff just because
        // jaxrs-api classes are on the classpath.
        if (JAXRSArchive.isJAXRS(archive)) {
            archive = archive.as(JAXRSArchive.class).staticContent();
            setup(archive);
        }
        return archive;
    }
}
